%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%DECLARATION OF ENDOGENOUS VARIABLES%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var 
    V U C H K S X I IS Sdash lambda Wh rK adash RPS LAMBDA R Q u au Rn PIE PIEw W Jw Hd MRSS JJw WO Deltaw YW A PWP MC Jp Y MCS JJp PO Deltap MPS G RK CEquiv
    dyobs dcobs pinfobs robs shadow_robs labobs dinvobs dwobs; // observables

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%DECLARATION OF EXOGENOUS VARIABLES%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varexo epsA epsG epsMCS epsMPS epsIS epsMRSS epsRPS;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%DECLARATION OF PARAMETERS%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters gy alp delta zzetap zzetaw PIE_bar g_bar  betta      // calibrated/fixed parameters
           sigma_c chi phiX xip xiw gammap gammaw pssi gamma2 F // estimated parameters: structural parameters
           rho_r theta_pie theta_y theta_dy                     // estimated parameters: Taylor rule
           rhoA rhoG rhoMCS rhoMRSS rhoMPS rhoRPS rhoIS         // estimated parameters: shock processes
           sigma_s;                                             // misc parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%CALIBRATED/FIXED OF PARAMETERS%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  gy           = 0.18;
  alp          = 0.67;
  delta        = 0.0250;
  zzetap       = 7.0;
  zzetaw       = 3.0;
  %PIE_bar      = 100*0.008662984461054; %estimated directly from data
  %g_bar        = 100*0.003550693291143; %estimated directly from data 
   PIE_bar       =100*0.00472907;  % Paul =1.008662984461054;
   g_bar         =100*0.002750617; %0.003550693291143;
  betta        = 0.99; % imposed and not estimated
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%ESTIMATED STRUCTURAL PARAMETERS%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  sigma_c      = 1.5;
  chi          = 0.7;
  phiX         = 2.0;
  xip          = 0.7;
  xiw          = 0.7;
  gammap       = 0.5;
  gammaw       = 0.5;
  pssi         = 2;
  gamma2       = 0.5;
  F            = 0.25;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%ESTIMATED POLICY PARAMETERS%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  rho_r        = 0.7;
  theta_pie    = 1.5;
  theta_y      = 0.3;
  theta_dy     = 0.3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%ESTIMATED SHOCK PROCESS PARAMETERS%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  rhoA         = 0.7;
  rhoG         = 0.7;
  rhoMCS       = 0.7;
  rhoMRSS      = 0.7;
  rhoMPS       = 0.7;
  rhoRPS       = 0.7;
  rhoIS        = 0.7;

%%%%%%%%%%%%%%%%%%%
%%MISC PARAMETERS%%
%%%%%%%%%%%%%%%%%%%

  sigma_s      = 0.001;
  
% ----------------------------
% *** DSGE-Model-equations ***
% ----------------------------
model;
// name = '(72) Calibration of beta)'
  # g                               = g_bar/100;
  
[ name = '(170) Intertemporal welfare']
  V                                 = U+betta*(1+g)^(1-sigma_c)*V(+1);

[ name = '(171) Period utility']
  U                                 = ((C-chi*C(-1)/(1+g))^(1-sigma_c))/(1-sigma_c)*exp((sigma_c-1)*H^(1+pssi)/(1+pssi));

[ name = '(172) Law of motion for capital']
  K                                 = (1-delta)*K(-1)/(1+g)+(1-S)*I*IS;

[ name = '(173) Investment']
  X                                 = I*(1+g)/I(-1);

[ name = '(174) Capital adjustment cost']
  S                                 = phiX*(X-1-g)^2;

[ name = '(175) Capital adjustment cost']
  Sdash                             = 2*phiX*(X-1-g);

[ name = '(176) Consumption foc with internal habit']
  lambda                            = (1-sigma_c)*U/(C-chi*C(-1)/(1+g))-betta*chi*(1+g)^(-sigma_c)*(1-sigma_c)*U(+1)/(C(+1)-chi*C/(1+g));

[ name = '(177) Household foc for hours']
  Wh                                = (C-chi*C(-1)/(1+g))*H^pssi/(1-betta*chi*(1+g)^(-sigma_c)*U(+1)/U*(C-chi*C(-1)/(1+g))/(C(+1)-chi*C/(1+g)));

[ name = '(178) Household foc for capital']
  rK                                = adash;

[ name = '(179) Euler equation']
  1                                 = RPS*R(+1)*LAMBDA;

[ name = '(180) Household foc for capital']
  Q                                 = LAMBDA(+1)*(rK(+1)*u(+1)-au(+1)+(1-delta)*Q(+1));

[ name = '(181) Household foc for investment']
  1                                 = Q*(1-S-Sdash*X)*IS
                                    + LAMBDA(+1)*Q(+1)*Sdash(+1)*X(+1)^2*IS(+1);

[ name = '(182) Stochastic discount factor']
  LAMBDA(+1)                            = betta*(1+g)^(-sigma_c)*lambda(+1)/lambda;

[ name = '(183) Ex-post real return']
  R                                 = Rn(-1)/PIE;

  # gamma1                          = 1/(betta*(1+g)^(-sigma_c))-(1-delta);
[ name = '(184) Capital utilisation adjustment function']
  au                                = gamma1*(u-1)+gamma2/(1-gamma2)*gamma1/2*(u-1)^2;

[ name = '(185) Capital utilisation adjustment function derivative']
  adash                             = gamma1+gamma2/(1-gamma2)*gamma1*(u-1);

[ name = '(186) Wage inflation']
  PIEw                              = (1+g)*W/W(-1)*PIE;

[ name = '(187) Wage setting condition']
  Jw                                = (1/(1-1/zzetaw))*Wh*Hd*MRSS
                                    + xiw*(1+g)*LAMBDA(+1)*(PIEw(+1)^zzetaw/PIE^(gammaw*zzetaw))*Jw(+1);

[ name = '(188) Wage setting condition']
  JJw                               = Hd + xiw*LAMBDA(+1)*(PIEw(+1)^zzetaw/(PIE^(gammaw*(zzetaw-1))*PIE(+1)))*JJw(+1);

[ name = '(189) Wage setting condition']
  WO                                = Jw/(W*JJw);

[ name = '(190) Law of motion for wage level']
  1                                 = xiw*(PIE(-1)^gammaw/PIEw)^(1-zzetaw)+(1-xiw)*WO^(1-zzetaw);

[ name = '(191) Law of motion for wage dispersion']
  Deltaw                            = xiw*(PIEw^zzetaw/PIE(-1)^(zzetaw*gammaw))*Deltaw(-1)+(1-xiw)*WO^(-zzetaw);

[ name = '(192) Production function']
  YW                                = (A*Hd)^alp*(u*K(-1)/(1+g))^(1-alp) - F*STEADY_STATE(YW);

[ name = '(193) Firm for for labour']
  W                                 = alp*PWP*(YW+F*STEADY_STATE(YW))/Hd;

[ name = '(194) Firm foc: return on capital']
  rK                                = (1-alp)*PWP*(YW+F*STEADY_STATE(YW))/(u*K(-1)/(1+g));

[ name = '(195) Marginal cost']
  MC                                = PWP;

[ name = '(196) Price setting condition']
  Jp                                = (1/(1-1/zzetap))*Y*MC*MCS
                                      +xip*(1+g)*LAMBDA(+1)*(PIE(+1)^zzetap/PIE^(gammap*zzetap))*Jp(+1);

[ name = '(197) Price setting condition']
  JJp                               = Y+xip*(1+g)*LAMBDA(+1)*(PIE(+1)^(zzetap-1)/PIE^(gammap*(zzetap-1)))*JJp(+1);

[ name = '(198) Price setting condition']
  PO                                = Jp/JJp;

[ name = '(199) Law of motion for price level']
  1                                 = xip*(PIE(-1)^(gammap)/PIE)^(1-zzetap)+(1-xip)*(PO)^(1-zzetap);

[ name = '(200) Law of motion for price dispersion']
  Deltap                            = xip*(PIE^zzetap/PIE(-1)^(zzetap*gammap))*Deltap(-1)+(1-xip)*PO^(-zzetap);

[ name = '(201) Taylor rule']
  log(Rn/STEADY_STATE(Rn))          = rho_r*log(Rn(-1)/STEADY_STATE(Rn))
                                    +(1-rho_r)*(theta_pie*log(PIE/STEADY_STATE(PIE))+theta_y*log(Y/STEADY_STATE(Y))+theta_dy*log(Y/Y(-1)))
                                    +log(MPS);

[ name = '(202) Resource constraint']
  Y                                 = C+G+I+au*K(-1)/(1+g);

[ name = '(203) Market clearing for labour)']
  H                                 = Deltaw*Hd;

[ name = '(204) Market clearing for goods']
  YW                                = Deltap*Y;

[ name = '(205) Gross real return on capital']
  RK                                = (rK*u-au+(1-delta)*Q)/Q(-1);

[ name = '(206) Law of motion for productivity']
  log(A)-log(STEADY_STATE(A))       = rhoA*(log(A(-1))-log(STEADY_STATE(A)))+epsA;

[ name = '(207) Law of motion for government expenditure']
  log(G)-log(STEADY_STATE(G))       = rhoG*(log(G(-1))-log(STEADY_STATE(G)))+epsG;

[ name = '(208) Law of motion for price markup shock']
  log(MCS)-log(STEADY_STATE(MCS))   = rhoMCS*(log(MCS(-1))-log(STEADY_STATE(MCS)))+epsMCS;

[ name = '(209) Law of motion for wage markup shock']
  log(MRSS)-log(STEADY_STATE(MRSS)) = rhoMRSS*(log(MRSS(-1))-log(STEADY_STATE(MRSS)))+epsMRSS;

[ name = '(210) Law of motion for capital quality shock']
  log(IS)-log(STEADY_STATE(IS))     = rhoIS*(log(IS(-1))-log(STEADY_STATE(IS)))+epsIS;

[ name = '(211) Law of motion for monetary policy shock']
  log(MPS)-log(STEADY_STATE(MPS))   = rhoMPS*(log(MPS(-1))-log(STEADY_STATE(MPS)))+epsMPS;

[ name = '(212) Law of motion for risk premium shock']
  log(RPS)-log(STEADY_STATE(RPS))   = rhoRPS*(log(RPS(-1))-log(STEADY_STATE(RPS)))+epsRPS;

[ name = '(214) Consumption equivalent variation']
  CEquiv                            = (1.01^(1-sigma_c)-1)*U + betta*(1+g)^(1-sigma_c)*CEquiv(+1);

[ name = '(64) Measurement equation for output growth']
  dyobs                             = log(Y/Y(-1)*(1+g));

[ name = '(65) Measurement equation for consumption growth']
  dcobs                             = log(C/C(-1)*(1+g));

[ name = '(66) Measurement equation for investment growth']
  dinvobs                           = log(I/I(-1)*(1+g));

[ name = '(67) Measurement equation for real wage growth']
  dwobs                             = log(W/W(-1)*(1+g));

[ name = '(68) Measurement equation for hours']
  labobs                            = (Hd-STEADY_STATE(Hd))/STEADY_STATE(Hd);

[ name = '(69) Measurement equation for policy rate']
  robs                              = Rn-1;

[ name = '(69) Measurement equation for policy rate']
  shadow_robs                       = Rn-1;

[ name = '(70) Measurement equation for inflation']
  pinfobs                           = log(PIE);
end;

shocks;
  var epsA;    stderr sigma_s;
  var epsG;    stderr sigma_s;
  var epsMCS;  stderr sigma_s ;
  var epsMPS;  stderr sigma_s ;
  var epsMRSS; stderr sigma_s ;
  var epsRPS;  stderr sigma_s ;
  var epsIS;   stderr sigma_s;
end;

%stoch_simul(order=1,irf=40)  Y C I H Hd W R Q Rn MC PIE PIEw u V Deltap  Deltaw;
stoch_simul(order=2,pruning, irf=40)  Y C I H Hd W R Q Rn MC PIE PIEw u V Deltap  Deltaw ;


check;
steady;

